Package: BMS
Type: Package
Title: Bayesian Model Averaging Library
Version: 0.3.5
Date: 2022-08-05
Author: Martin Feldkircher and Stefan Zeugner and Paul Hofmarcher
Maintainer: Stefan Zeugner <stefan.zeugner@ec.europa.eu>
Depends: methods, stats, graphics, R (>= 2.10)
Description: Bayesian Model Averaging for linear models with a wide choice of (customizable) priors. Built-in priors include coefficient priors (fixed, hyper-g and empirical priors), 5 kinds of model priors, moreover model sampling by enumeration or various MCMC approaches. Post-processing functions allow for inferring posterior inclusion and model probabilities, various moments, coefficient and predictive densities. Plotting functions available for posterior model size, MCMC convergence, predictive and coefficient densities, best models representation, BMA comparison. Also includes Bayesian normal-conjugate linear model with Zellner's g prior, and assorted methods.
License: BSD_3_clause + file LICENSE
URL: http://bms.zeugner.eu
RoxygenNote: 7.2.1
LazyData: true
NeedsCompilation: no
Packaged: 2022-08-05 10:42:16 UTC; admin-b1
Repository: CRAN
Date/Publication: 2022-08-09 11:10:02 UTC
Built: R 4.3.3; ; 2025-01-24 04:54:38 UTC; unix
