# Generated by roxygen2: do not edit by hand

importFrom(stats,density)
importFrom(stats,quantile)
importFrom(stats,variable.names)
importFrom(stats,vcov)
importFrom(stats,predict)
importFrom(stats,deviance)
importFrom(stats,coef)
importFrom(stats,model.frame)
importFrom("methods", "is", "new")
importFrom("stats", "as.formula", "cor", "na.omit", "var")

S3method("[",bma)
S3method("[",topmod)
S3method(c,bma)
S3method(coef,bma)
S3method(density,bma)
S3method(density,zlm)
S3method(deviance,bma)
S3method(deviance,zlm)
S3method(image,bma)
S3method(logLik,zlm)
S3method(model.frame,bma)
S3method(plot,bma)
S3method(plot,pred.density)
S3method(predict,bma)
S3method(predict,zlm)
S3method(print,bma)
S3method(print,pred.density)
S3method(print,topmod)
S3method(quantile,coef.density)
S3method(quantile,density)
S3method(quantile,pred.density)
S3method(summary,bma)
S3method(summary,zlm)
S3method(variable.names,bma)
S3method(variable.names,zlm)
S3method(vcov,zlm)
export(as.zlm)
export(beta.draws.bma)
export(bin2hex)
export(bms)
export(estimates.bma)
export(f21hyper)
export(fullmodel.ssq)
export(gdensity)
export(hex2bin)
export(info.bma)
export(is.bma)
export(is.topmod)
export(lps.bma)
export(plotComp)
export(plotConv)
export(plotModelsize)
export(pmp.bma)
export(pmpmodel)
export(post.pr2)
export(post.var)
export(pred.density)
export(topmod)
export(topmodels.bma)
export(zlm)
importFrom(stats,density)
