#' @details
#' The key function you need is  \code{\link{bms}}.  
#' @author Martin Feldkircher, Paul Hofmarcher, and Stefan Zeugner
#' @seealso \code{\link{coef.bma}}, \code{\link{plotModelsize}} and
#' \code{\link{density.bma}} for some operations on the resulting 'bma' object, 
#' as well as 
#' \code{\link{predict.bma}} or \code{\link{gdensity}}, or
#' \code{\link{zlm}} for individual Zellner regression models.
#' 
#' Check \url{http://bms.zeugner.eu} for additional help.
#' @references 
#' \url{http://bms.zeugner.eu}: BMS package homepage with help and tutorials
#' 
#' Feldkircher, M. and S. Zeugner (2015): Bayesian Model Averaging Employing 
#' Fixed and Flexible Priors: The BMS Package for R, Journal of Statistical Software 68(4).
#' 
#' Feldkircher, M. and S. Zeugner (2009): Benchmark Priors
#' Revisited: On Adaptive Shrinkage and the Supermodel Effect in Bayesian Model
#' Averaging, IMF Working Paper 09/202.
#' 
#' @keywords internal
"_PACKAGE"